(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "gigantes"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.html = exports.loadDomSelector = void 0;
    const gigantes_1 = require("gigantes");
    exports.loadDomSelector = (path) => {
        const module = require(path);
        const domSelectors = [];
        Object.keys(module).forEach((key) => {
            if (["SingleDomSelector", "MultipleDomSelector"].includes(module[key].constructor.name)) {
                domSelectors.push({
                    name: key,
                    domSelector: module[key],
                });
            }
        });
        return domSelectors;
    };
    const applyDomNode = (body, data, fn) => {
        const domNodes = data.domSelector.getDomNode()(body);
        if (Array.isArray(domNodes)) {
            gigantes_1.nestMap(domNodes, (domNode) => {
                fn(domNode, data.name);
            });
        }
        else {
            fn(domNodes, data.name);
        }
    };
    exports.html = (filePath, url, domSelectorName) => {
        try {
            const $ = gigantes_1.fetchSync(url);
            const body = new gigantes_1.DomNode($("body"), $);
            $("head").append(`<style>
    .gigantes-focus {
        position: relative;
        border: solid 10px #ff0099;
    }
    .domselector-name {
        position: absolute;
        top: -40px;
        background-color: lightgray;
        padding: 2px 4px;
        color: black;
        left: -10px;
    }
    </style>`);
            const domSelector = exports.loadDomSelector(filePath).find(({ name }) => name == domSelectorName);
            applyDomNode(body, domSelector, (domNode, name) => {
                domNode.cheerio.addClass("gigantes-focus");
                domNode.cheerio.append(`<div class="domselector-name">${name}</div>`);
            });
            return $.html();
        }
        catch {
            return "";
        }
    };
});
